package algae.simulation;

import java.io.*;
import algae.simulation.Settings;
import algae.simulation.worlds.AgentWorld;
import algae.simulation.agents.Agent;

public final class SimulationStats {
	
	File file;
	String fileName;
	FileOutputStream stream;
	OutputStreamWriter ostream;
	
	AgentWorld aw;
	Agent tmp;
	 	   
	Settings cfg;
	
	public SimulationStats(String s)
	{
	       fileName=s;
	       file=new File(s);
	}
	
	public boolean openStream(Settings set, AgentWorld agw)
	{
	       aw=agw;
	       cfg=set;
	       try {
	           stream=new FileOutputStream(file);
	           ostream = new OutputStreamWriter(stream);
	           
	           ostream.write("[SECTION SETTINGS]\n\n");
	           
	           tmp=(Agent)aw.agents.elementAt(0);
	           ostream.write("<SUBSECTION SIMULATION>\n");
	           ostream.write("steps = "+cfg.duration+"\n");
	           ostream.write("random_seed = "+cfg.randSeed);
	           ostream.write("world_width = "+cfg.width+"\n");
	           ostream.write("world_height = "+cfg.height+"\n");
	           ostream.write("neurons_used = "+tmp.phenoType.nCount+"\n");
	           ostream.write("input_neurons_used = "+tmp.phenoType.inCount+"\n");
	           ostream.write("output_neurons_used = "+tmp.phenoType.outCount+"\n");
	           ostream.write("initial_agents = "+cfg.initial_algaes+"\n");
	           ostream.write("initial_energy = "+cfg.initial_energy+"\n");
	           ostream.write("substrate = "+cfg.substrate_amount+"\n");
	           ostream.write("substrate_renew_rate = "+cfg.substrate_renew_rate+"\n");
	           ostream.write("pheromone_decrease_rate = "+cfg.pheromone_dec_rate+"\n");
                   ostream.write("pheromone_sensor_fov = "+cfg.pheromone_fov+"\n");
	           ostream.write("pheromone_sensor_range = "+cfg.pheromone_range+"\n");
	           ostream.write("substrate_sensor_fov = "+cfg.substrate_fov+"\n");
	           ostream.write("substrate_sensor_range = "+cfg.substrate_range+"\n");
	           ostream.write("light_sensor_fov = "+cfg.light_fov+"\n");
	           ostream.write("light_sensor_range = "+cfg.light_range+"\n");
	           ostream.write("bumper_sensor_range = "+cfg.bumper_range+"\n");
	           ostream.write("\n");	      
	 	
	 	   for (int z=0; z<aw.agentCount; z++)
	 	   {
	 	   	tmp=(Agent)aw.agents.elementAt(z);
	 	   	
	 	   	ostream.write("<SUBSECTION AGENT #"+z+">\n");
	 	   	ostream.write("agent_index = "+z+"\n");
	 	   	ostream.write("phenotype_miller_matrix\n");
	 	   	
	 	   	ostream.write("position_x = "+tmp.x+"\n");
	 	   	ostream.write("position_y = "+tmp.y+"\n");
	 	   	ostream.write("energy_level = "+tmp.energy_level+"\n");
	 	   	
	 	   	// stream out the phenotype
	 	   	 	   	
	 	   	for (int i=0; i<50; i++)
	 	   	{
	 	   	    for (int j=0; j<50; j++)
	 	   	    {
	 	   	    	ostream.write(tmp.phenoType.neuron_matrix[i][j]+" ");
	 	   	    }
	 	   	    ostream.write("\n");
	 	   	}
	 	   	
	 	   	ostream.write("phenotype_weight_matrix\n");
	 	   	
	 	   	for (int i=0; i<50; i++)
	 	   	{
	 	   	    for (int j=0; j<50; j++)
	 	   	    {
	 	   	    	ostream.write(tmp.phenoType.weight_matrix[i][j]+" ");
	 	   	    }
	 	   	    ostream.write("\n");
	 	   	}
	 	   	
	 	   	ostream.write("phenotype_spline_matrix\n");
	 	   	
	 	   	for (int i=0; i<2; i++)
	 	   	{
	 	   	    for (int j=0; j<10; j++)
	 	   	    {
	 	   	    	ostream.write(tmp.phenoType.spline_matrix[i][j]+" ");
	 	   	    }
	 	   	    ostream.write("\n");
	 	   	}	 	   		
	 	   	
	 	        ostream.write("\n");	      
	 	        ostream.flush();
	 	   }
	 	   ostream.write("\n");	      
	 	
	 	   ostream.flush();
	           
	       } catch (IOException e) 
	       {
	       	   return false;
	       }
	       cfg=set;
	       
	       return true;
	}
		
	public boolean writeStats(int stepNo)
	{
	       try {
                   ostream.write("[SECTION STEP "+stepNo+"]\n");
                   
                   ostream.write("<SUBSECTION GENERAL STATS>\n");
                   ostream.write("agents_alive = "+aw.aliveAgents()+"\n");
                   ostream.write("agents_died = "+aw.stepDeadCount+"\n");  
                   ostream.write("mitosis_count = "+aw.stepMitosisCount+"\n");  
                   ostream.write("meiosis_count = "+aw.stepMeiosisCount+"\n");  
                   ostream.write("light_intensity = "+aw.lw.intensity+"\n");
                   ostream.write("substrate_amount = "+aw.sw.amount+"\n");
                   ostream.write("substrate_consumed = "+aw.sw.used+"\n");
                   ostream.write("substrate_renewed = "+aw.sw.renew+"\n");
                   ostream.write("pheromone_dissolved = "+aw.pw.used+"\n");
                   ostream.write("pheromone_renewed = "+aw.pw.renew+"\n");
                   ostream.write("light_consumed = "+aw.lw.used+"\n\n");
                   
                   for (int z=0; z<aw.agentCount; z++)
	 	   {
	 	   	tmp=(Agent)aw.agents.elementAt(z);
	 	   
	 	   	if (!tmp.isAlive()) continue;
	 	   	
	 	   	ostream.write("<SUBSECTION AGENT #"+z+">\n");
	 	   	ostream.write("agent_index = "+z+"\n");
	 	   	ostream.write("heading = "+tmp.dir+"\n");
	 	   	ostream.write("position_x = "+tmp.x+"\n");
	 	   	ostream.write("position_y = "+tmp.y+"\n");
	 	   	ostream.write("energy_level = "+tmp.energy_level+"\n");
	 	   	ostream.write("ann_output = "+tmp.ann_output+"\n");
	 	   	ostream.write("mitosis = "+tmp.madeMitosis+"\n");
	 	   	ostream.write("meiosis = "+tmp.madeMeiosis+"\n");
	 	   	ostream.write("used_light = "+tmp.usedLight+"\n");
	 	   	ostream.write("used_substrate = "+tmp.usedSubstrate+"\n");
	 	   	ostream.write("died = "+tmp.died+"\n");
	 	   	ostream.write("birth = "+tmp.born+"\n");
	 	   	ostream.write("sensors_values\n");
	 	   	
	 	   	for (int i=0; i<tmp.SENSORCOUNT; i++)
	 	   	{
	 	   	     ostream.write(tmp.sensor_values[i]+" ");
	 	   	}
	 	   	ostream.write("\n");
	 	   		 	   	
	 	   	ostream.write("phenotype_miller_matrix\n");
	 	   	
	 	   	// stream out the phenotype
	 	   	for (int i=0; i<50; i++)
	 	   	{
	 	   	    for (int j=0; j<50; j++)
	 	   	    {
	 	   	    	ostream.write(tmp.phenoType.neuron_matrix[i][j]+" ");
	 	   	    }
	 	   	    ostream.write("\n");
	 	   	}
	 	   	
	 	   	ostream.write("phenotype_weight_matrix\n");
	 	   	
	 	   	for (int i=0; i<50; i++)
	 	   	{
	 	   	    for (int j=0; j<50; j++)
	 	   	    {
	 	   	    	ostream.write(tmp.phenoType.weight_matrix[i][j]+" ");
	 	   	    }
	 	   	    ostream.write("\n");
	 	   	}
	 	   	
	 	   	ostream.write("phenotype_spline_matrix\n");
	 	   	
	 	   	for (int i=0; i<2; i++)
	 	   	{
	 	   	    for (int j=0; j<10; j++)
	 	   	    {
	 	   	    	ostream.write(tmp.phenoType.spline_matrix[i][j]+" ");
	 	   	    }
	 	   	    ostream.write("\n");
	 	   	}	 	   		
	 	   	
	 	        ostream.write("\n");	      
	 	        ostream.flush();
	 	   }
	 	   ostream.write("\n");	      
	 	                  
                   ostream.flush();
	           
	       } catch (IOException e)
	       {
	       	     return false;
	       }
	       return true;
	}
	
	public boolean closeStream()
	{
	       try {
	       	   ostream.flush();
	           stream.close();
	       } catch (IOException e)
	       {
	       	   return false;
	       }
	       return true;
	}
}