package algae.simulation.agents;

import java.util.*;
import java.lang.*;
import java.lang.Object;
import java.lang.Math;
import java.lang.ArrayIndexOutOfBoundsException;

public class PhenoType {
	
    public int nCount = 50; // count of neurons used
    
    public int inCount = 17; // number of input-neurons
    public int outCount = 5; // number of output-neurons

    int inOutCount = inCount+outCount;
    int hiddenCount = nCount-inOutCount;

    // neurons from 1 - 17 (resp. 0-16) = input neurons	
    // neurons from 18 - 22 (resp. 17-21) = output neurons
    // rest of it = hidden layer neurons
    public int neuron_matrix[][] = new int[nCount][nCount]; // [from][to]
    public float weight_matrix[][] = new float[nCount][nCount];
    public float spline_matrix[][] = new float[2][10];

    public float neuronOutput[] = new float[nCount];
    public boolean visited[] = new boolean[nCount];
    
    public CubicSpline cs;
    public Vector x;
    public Vector y;
        
    public PhenoType(GenoType g){
		
    	// extract neuron matrix (miller matrix) out of genotype
    	for (int i=0; i<nCount; i++)
    	    for (int j=0; j<nCount; j++)
    	    	if (g.nm_bit.get(j*50+i)==true)
    	    	    neuron_matrix[i][j]=1;
    	    	else
    	    	    neuron_matrix[i][j]=0;    	    		
    	
    	// extract weight matrix out of genotype   
    	for (int i=0; i<nCount; i++)
    	    for (int j=0; j<nCount; j++)
    	    	weight_matrix[i][j]=g.getFloatOuttaWm(j*50+i);
    	    	                         	
    	// extract splines out of genotype
    	for (int i=0; i<10; i++)
    	{
    	    spline_matrix[0][i]=g.getFloatOuttaSm(i*2);
    	    spline_matrix[1][i]=g.getFloatOuttaSm(i*2+1);
    	}
    	    
    	Vector x=new Vector(spline_matrix[0].length); 
    	Vector y=new Vector(spline_matrix[0].length);    	

    	// construct vectors used by splines
    	for (int z=0; z<spline_matrix[0].length; z++)
    	{
    	    x.add(z,new Double(spline_matrix[0][z]));
    	    y.add(z,new Double(spline_matrix[1][z]));
    	}   	
    	cs=new CubicSpline(spline_matrix[0].length,x,y);
    }
        
    public boolean connected(int i, int j)
    {
    	return (neuron_matrix[i][j]==1);
    }    
                
    public float transferFunc(int funcSel, float val)
    {
    	// TODO: we don't care of the funcSel (not this time -> future version)
    	/*
    	switch (funcSel){
	case 0: return (float)Math.sin((double)val);
	case 1: return (float)Math.cos((double)val);
	case 2: return (float)Math.tan((double)val);
	}
	*/
	
	//System.out.println("Value: "+val);
	// return cubic spline interpolated value
	
	float res=0.0f;
	
	try
	{
	   res=(float)cs.getValue(Math.abs((double)val));
	} catch (ArrayIndexOutOfBoundsException e)
	{
	   //System.out.println("EXCEPTION: Value == "+val);
	   return 0.0f;
	}
		
	return res;
    }
    
    // get number of parent nodes
    public int getPrevNum(int n)
    {
    	int cnt=0;
    	for (int z=0; z<nCount; z++)
    	    if ((neuron_matrix[z][n]==1)&&(z!=n))
    	    	cnt++;
    	return cnt;
    }
    
    // get Nth parent node 
    public int getPrev(int n)
    {
    	for (int z=0; z<nCount; z++)
    	    if ((neuron_matrix[z][n]==1)&&(z!=n)&&(!visited[z]))
    	    	return z;    	    	
    	return -1;
    }
    
    // calculate output for neuron n
    public float calculateNeuron(int n)
    {
    	float result=0.0f;
    	int prevNum = getPrevNum(n);
    	
    	for (int i=0; i<nCount; i++)
    	     visited[i]=false;

    	for (int z=0; z<prevNum; z++)
    	{ 
    	     int prev=getPrev(n);

    	     // TODO: well, we don't really select function 111 ;) 
             // it's only a dummy var for future extensions
    	     result+=transferFunc(111,neuronOutput[prev]*weight_matrix[prev][n]);	     
    	     visited[prev]=true;
    	}
    	return result;
    }
    
    // process current neural network
    // and return the number of the winner neuron (the winner takes all)
    public int processNet(float inVec[])
    {
    	int i=0,j=0,z=0;
    	float out=0.0f;
    	
    	// fill input vector with input data (2BDone correctly)
    	for (z=0; z<inCount; z++)
            neuronOutput[z]=inVec[z];    	     
        
        // reset other outputs 
        for (z=inCount; z<nCount; z++)
            neuronOutput[z]=0.0f;    
        
        // calculate all outputs for neurons from top down
        for (z=inOutCount; z<nCount; z++)
        {
            if (neuron_matrix[z][z]!=0) // is neuron active ?
            {
            	neuronOutput[z]=calculateNeuron(z);
            	out+=neuronOutput[z];
            }
            // TODO: Noch irgendwo checken, ob Verbindung
            // zu jedem Outputneuron, ansonsten auf 1 setzen !!!
        }

	int max=0;  
	// calculate outputs and look out for winner neuron too      
        for (z=inCount; z<inOutCount; z++)
        {
             neuronOutput[z]=calculateNeuron(z);
             if (neuronOutput[z]>neuronOutput[max])
             	max=z;
             out+=neuronOutput[z];
        }
        
        if (max==0) 
           return 0;             	
        else
    	   return (max-inCount);
    }
    
    public void printNeuronMatrix()
    {
    	for (int i=0; i<nCount; i++)
    	{
    	    for (int j=0; j<nCount; j++)
    	    {
    	    	if (neuron_matrix[i][j]!=1)
    	    	    System.out.print(" ");
    	    	else
    	    	    System.out.print(neuron_matrix[i][j]);
    	    }
    	    System.out.print("\n");	
    	}    	
    }
    
    public void printWeightMatrix()
    {
    	for (int i=0; i<nCount; i++)
    	{
    	    for (int j=0; j<nCount; j++)
    	    {
    	    	System.out.print(weight_matrix[i][j]);
    	    }
    	    System.out.print("\n");	
    	}    	
    }
    
    public void printOutput()
    {
    	for (int z=inCount; z<inOutCount; z++)
    	{
    	    System.out.println("OutputNeuron #"+(z-inCount+1)+" = "+neuronOutput[z]);
    	}
    }
}
